import { defineConfig } from "@hey-api/openapi-ts";

export default defineConfig({
  input: "./generated/openapi.json",
  output: "./generated/api",
  plugins: [
    "@hey-api/typescript",
    "@hey-api/schemas",
    "@hey-api/sdk",
    "@hey-api/client-nuxt",
{%- if generate_zod %}
    "zod",
{%- endif %}
  ],
});
