<!DOCTYPE html>
<html>

<head>
    <meta charset="utf-8" />
    <!--IE compatibility-->
    <meta http-equiv="X-UA-Compatible" content="IE=edge" />
    <meta name="viewport" content="width=device-width, initial-scale=1.0, maximum-scale=1.0" />
</head>

<body>
    <div id="app" data-page="{{ page | escape }}"></div>
</body>

</html>
