# Dependencies
node_modules/

# Build output
{{ bundle_dir }}/
dist/

# Vite
*.local
.vite/

# TypeScript
*.tsbuildinfo

# Generated types
{{ resource_dir }}/generated/

# IDE
.idea/
.vscode/
*.swp
*.swo

# OS
.DS_Store
Thumbs.db

# Logs
logs/
*.log
npm-debug.log*

# Environment
.env
.env.local
.env.*.local

# Python
__pycache__/
*.py[cod]
.venv/
venv/
