{
  "name": "{{ project_name }}",
  "version": "0.0.0",
  "private": true,
  "type": "module",
  "scripts": {
    "ng": "ng",
    "start": "ng serve --proxy-config proxy.conf.json",
    "build": "ng build",
    "serve": "npx serve dist/{{ project_name }}/browser",
    "watch": "npm run serve",
    "update": "npm update"
  },
  "dependencies": {
{%- for dep in dependencies %}
    "{{ dep }}": "latest"{% if not loop.last %},{% endif %}
{%- endfor %}
  },
  "devDependencies": {
{%- for dep in dev_dependencies %}
    "{{ dep }}": "latest",
{%- endfor %}
{%- if generate_client %}
    "@hey-api/openapi-ts": "^0.88.0",
{%- endif %}
    "@types/jasmine": "latest"
  }
}
