<!DOCTYPE html>
<html lang="en">
  <head>
    <meta charset="UTF-8" />
    <link rel="icon" type="image/svg+xml" href="/static/vite.svg" />
    <meta name="viewport" content="width=device-width, initial-scale=1.0" />
    <title inertia>{{ project_name }}</title>
    {{ "{{ vite_hmr() }}" }}
    {{ "{{ vite('{{ resource_dir }}/main.ts') }}" }}
  </head>
  <body>
    <div id="app" data-page='{{ "{{ inertia | safe }}" }}'></div>
  </body>
</html>
