{% raw %}{% extends "base.html.j2" %}{% endraw %}

{% raw %}{% block content %}{% endraw %}
<main class="app">
  <h1>{{ "{{ project_name }}" }}</h1>
  <div class="card">
    <button hx-get="/api/hello" hx-target="#greeting" hx-swap="innerHTML">
      Load Greeting from API
    </button>
    <p id="greeting"></p>
  </div>
</main>
{% raw %}{% endblock %}{% endraw %}
