<script setup lang="ts">
const count = ref(0);

function increment() {
  count.value += 1;
}
</script>

<template>
  <div class="app">
    <h1>{{ "{{ project_name }}" }}</h1>
    <p>Nuxt 3 + Litestar</p>
    <div class="card">
      <button @click="increment">
        count is {{ "{{ count }}" }}
      </button>
    </div>
  </div>
</template>

<style scoped>
.app {
  max-width: 1280px;
  margin: 0 auto;
  padding: 2rem;
  text-align: center;
}

.card {
  padding: 2em;
}

button {
  border-radius: 8px;
  border: 1px solid transparent;
  padding: 0.6em 1.2em;
  font-size: 1em;
  font-weight: 500;
  font-family: inherit;
  background-color: #1a1a1a;
  cursor: pointer;
  transition: border-color 0.25s;
}

button:hover {
  border-color: #00dc82;
}

@media (prefers-color-scheme: light) {
  button {
    background-color: #f9f9f9;
  }
}
</style>
