"""SPA mode handler public API.

This package provides the AppHandler for serving single-page applications
with Vite asset integration.
"""

from litestar_vite.handler._app import AppHandler

__all__ = ("AppHandler",)
