{% if use_tailwind %}@import "tailwindcss";

{% endif %}:root {
  font-family: "Inter", system-ui, -apple-system, BlinkMacSystemFont, "Segoe UI", sans-serif;
  line-height: 1.5;
  color: #0f172a;
  background: #f8fafc;
  min-height: 100vh;
}
