// This file is auto-generated by TanStack Router plugin
// Run `npm run dev` to regenerate this file

import { createRootRoute, createRoute } from "@tanstack/react-router";

// Placeholder route tree - will be regenerated by @tanstack/router-plugin
export const routeTree = createRootRoute();
