{
  "compilerOptions": {
    "target": "ES2022",
    "useDefineForClassFields": false,
    "module": "ESNext",
    "moduleResolution": "Bundler",
    "types": ["vite/client"],
    "allowImportingTsExtensions": true,
    "noEmit": true,
    "strict": true,
    "skipLibCheck": true,
    "forceConsistentCasingInFileNames": true,
    "resolveJsonModule": true,
    "baseUrl": ".",
    "paths": {
      "@/*": ["./{{ resource_dir }}/*"]
    }
  },
  "include": ["{{ resource_dir }}/**/*.ts"]
}
