<!DOCTYPE html>
<html lang="en">
  <head>
    <meta charset="UTF-8" />
    <meta name="viewport" content="width=device-width, initial-scale=1.0" />
    <title>{{ project_name }}</title>
  </head>
  <body>
    <div id="app"></div>
    <script type="module" src="/{{ resource_dir }}/main.tsx"></script>
  </body>
</html>
