import { bootstrapApplication } from "@angular/platform-browser";

import { AppComponent } from "./app/app.component";
import { appConfig } from "./app/app.config";
{% if use_tailwind %}import "./styles.css";{% endif %}

bootstrapApplication(AppComponent, appConfig).catch((err) =>
  console.error("Bootstrap failed:", err)
);
