{
  "compilerOptions": {
    "target": "ES2022",
    "useDefineForClassFields": true,
    "module": "ESNext",
    "lib": ["ES2022", "DOM", "DOM.Iterable"],
    "skipLibCheck": true,

    /* Bundler mode */
    "moduleResolution": "bundler",
    "allowImportingTsExtensions": true,
    "isolatedModules": true,
    "moduleDetection": "force",
    "noEmit": true,
{% if framework in ["react", "react-inertia", "react-router", "react-tanstack"] %}
    "jsx": "react-jsx",
    "types": ["vite/client"],
{% elif framework == "vue" or framework == "vue-inertia" %}
    "jsx": "preserve",
{% endif %}

    /* Linting */
    "strict": true,
    "noUnusedLocals": true,
    "noUnusedParameters": true,
    "noFallthroughCasesInSwitch": true,
    "noUncheckedSideEffectImports": true,

    /* Paths */
    "baseUrl": ".",
    "paths": {
      "@/*": ["./{{ resource_dir }}/*"]
    }
  },
  "include": ["{{ resource_dir }}/**/*", "vite.config.ts"{% if framework == "vue" or framework == "vue-inertia" %}, "env.d.ts"{% endif %}],
  "exclude": ["{{ resource_dir }}/generated/**"]
}
