:root {
  font-family: Inter, system-ui, Avenir, Helvetica, Arial, sans-serif;
  line-height: 1.5;
  font-weight: 400;

  color-scheme: light dark;
  color: rgba(255, 255, 255, 0.87);
  background-color: #242424;

  font-synthesis: none;
  text-rendering: optimizeLegibility;
  -webkit-font-smoothing: antialiased;
  -moz-osx-font-smoothing: grayscale;
}

.app {
  max-width: 1280px;
  margin: 0 auto;
  padding: 2rem;
  text-align: center;
}

.message {
  color: #646cff;
  font-weight: 500;
}

.back-link {
  color: #646cff;
  text-decoration: none;
}

.back-link:hover {
  text-decoration: underline;
}

.user-list {
  list-style: none;
  padding: 0;
  margin-top: 2rem;
}

.user-item {
  padding: 1rem;
  margin: 0.5rem 0;
  background: #1a1a1a;
  border-radius: 8px;
  display: flex;
  justify-content: space-between;
}

.user-item strong {
  color: #fff;
}

.user-item span {
  color: #888;
}

@media (prefers-color-scheme: light) {
  :root {
    color: #213547;
    background-color: #ffffff;
  }
  .user-item {
    background-color: #f9f9f9;
  }
}
